% Stock Market Prediction based on Wavelet and Neural Networks
clc;clear;close all;
% Load input sequences
load ('sequences.dat','-mat');
% Now "seq" is a cell and all input sequences are loaded (626 sequences of
% intraday prices, corresponding to 626 days of effective days of trading).
% Stock market data have been downloaded at http://www.ccg.it
% seq{ii} is a sequence of each single day of trading.
L = length(seq);
% Now L is the number of days
% I consider a random permutation of days
ordine = randperm(L);
% I consider 300 days for testing
Ntrain = 300;
% I consider the remaining days for testing
Ntest  = L-(Ntrain);
% From training days I consider only 10 sequences randomly selected,
% i.e. a total of 10x300 = 3000 training sequences
point_train = 10;
% From testing days I consider 130 sequences randomly selected,
% i.e. a total of 130 x (L-300) testing sequences
point_test  = 130;
% Simulation parameters
N         = 32;  % length of input sequences
M         = 3;   % length of predicted sequences
buysignal = 1;   % Set this value to 0 if you want Sell Signals, 1 for Buy Signals
%--------------------------------------------------------------------------
%----------------------------------------------------------------- TRAINING
% Memory allocation for training input sequences and target sequences
training_input  = zeros(N,Ntrain*point_train);
training_target = zeros(1,Ntrain*point_train);
% Auxiliary counter
cont = 1;
for ii=1:Ntrain
    % I consider a random day using the random permutation
    k     = ordine(ii);
    % Current day of trading
    myseq = seq{k};
    % Length of vector
    ls    = length(myseq);
    % From current day I extract #point_train sequences, randomly selected
    Ocurrent = randperm(ls-N-(M+1));
    % In this way I am sure that there input sequences have a length exactly
    % equal to N. Such sequences are randomly selected within current day.
    Ocurrent = Ocurrent+N;
    for jj=1:point_train
        x1    = Ocurrent(jj);
        x0    = x1-N+1;
        v     = myseq(x0:x1);
        v     = v(:);
        % Input sequence is added
        training_input(:,cont)  = v;
        vout                    = myseq(x1+1:x1+M);
        % Target is added
        if buysignal==1
            % myseq(x1) is current price (last price of input sequence)
            training_target(:,cont) = any(vout(:)>myseq(x1));
        else
            % myseq(x1) is current price (last price of input sequence)
            training_target(:,cont) = any(vout(:)<myseq(x1));
        end
        % Counter is updated
        cont           = cont+1;
    end
end

% Train neural networks
[netext]  = trainsystem(training_input,training_target,buysignal);
%--------------------------------------------------------------------------
%------------------------------------------------------------------ TESTING
testing_input  = zeros(N,Ntest*point_test);
testing_target = zeros(1,Ntest*point_test);
% Auxiliary counter
cont = 1;
for ii=Ntrain+1:Ntrain+Ntest
    % For testing I consider a random day using the random permutation
    % Note that there is no overlapp between training and testing days
    k     = ordine(ii);
    % Current day of trading
    myseq = seq{k};
    % Length of vector
    ls    = length(myseq);
    % From current day I extract #point_train sequences, randomly selected
    Ocurrent = randperm(ls-N-(M+1));
    % In this way I am sure that there input sequences have a length exactly
    % equal to N. Such sequences are randomly selected within current day.
    Ocurrent = Ocurrent+N;
    for jj=1:point_test
        x1    = Ocurrent(jj);
        x0    = x1-N+1;
        v     = myseq(x0:x1);
        v     = v(:);
        % Input sequence is added
        testing_input(:,cont)  = v;
        vout                   = myseq(x1+1:x1+M);
        % Target is added
        if buysignal==1
            % myseq(x1) is current price (last price of input sequence)
            testing_target(:,cont) = any(vout(:)>myseq(x1));
        else
            % myseq(x1) is current price (last price of input sequence)
            testing_target(:,cont) = any(vout(:)<myseq(x1));
        end
        % Counter is updated
        cont           = cont+1;
    end
end
% Simulate with testing patterns
[result] = simsystem(netext,testing_input);
%--------------------------------------------------------------------------
%---------------------------------------------------------- DISPLAY RESULTS
% System performances
recrate = nnz(result & testing_target)/nnz(result);
disp('Recognition rate');
disp(recrate);

